# app.py
#
# author: deng
# date  : 20250604

import streamlit as st
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate
from langchain_ollama import ChatOllama
from utils import parse_config


class TranslatorApp:
    """Streamlit App for Language Translation"""

    def __init__(self, config_path: str = 'config.yaml'):
        self._config = parse_config(config_path)
        self._chain = self._prepare_chain()

    def _prepare_chain(self) -> LLMChain:
        """Prepare the chain for translation"""
        template = (
            '你是專業的翻譯人員，請判斷這段句子「{input_text}」的語言是否為{source_lang}，若非的'
            '話則請回傳一模一樣的句子，若是的話則請將該句翻譯成{target_lang}，並且符合{description}'
            '（僅回傳翻譯結果即可）。'
        )
        llm = ChatOllama(
            base_url=self._config['app']['ollama_url'],
            model=self._config['app']['ollama_model_name'],
            temperature=self._config['app']['ollama_temperature'],
            max_tokens=self._config['app']['ollama_max_tokens'],
            top_p=self._config['app']['ollama_top_p'],
            keep_alive=self._config['app']['ollama_keep_alive'],
            stop=None
        )
        prompt = PromptTemplate(
            input_variables=['input_text', 'source_lang', 'target_lang', 'description'],
            template=template
        )
        return prompt | llm

    def run(self) -> None:
        """ Run the Streamlit app """

        st.set_page_config(
            page_title=self._config['app']['page_title'],
            page_icon=self._config['app']['page_favicon_path'],
        )
        st.title(body=self._config['app']['page_title'])

        direction = st.radio(
            label='語言',
            options=list(self._config['app']['lang_directions'].keys()),
            index=0,
            key='lang_choice',
            horizontal=True
        )

        input_text = st.text_area(
            label='輸入',
            placeholder='請輸入文字',
            key='input_text'
        )

        output_container = st.empty()

        if st.button('翻譯'):
            if not input_text.strip():
                st.warning("請輸入要翻譯的文字")
                return

            with st.spinner('翻譯中...'):

                result = self._chain.stream({
                    'input_text': input_text,
                    'source_lang': self._config['app']['lang_directions'][direction]['source_lang'],
                    'target_lang': self._config['app']['lang_directions'][direction]['target_lang'],
                    'description': self._config['app']['lang_directions'][direction]['description']
                })

            output_container.write_stream(
                stream=result
            )


if __name__ == '__main__':
    app = TranslatorApp()
    app.run()
